// BasicTab.cpp : implementation file
//

#include "stdafx.h"
#include "SensorSim.h"
#include "BasicTab.h"
#include "SensorSimDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CSensorSimDlg*  pSensorSim;
/////////////////////////////////////////////////////////////////////////////
// CBasicTab dialog


CBasicTab::CBasicTab(CWnd* pParent /*=NULL*/)
	: CDialog(CBasicTab::IDD, pParent)
{
	//{{AFX_DATA_INIT(CBasicTab)
	m_blackCountType = 1;
	m_blackRatioWhite = _T("0.2");
	m_blackNumber = _T("36");
	m_planeRadius = _T("");
	m_planeType = 0;
	m_planeX = _T("3");
	m_planeY = _T("3");
	m_whiteLandFail = _T("0");
	m_whiteNumber = _T("100");
	m_whiteTransmRange = _T("0.5");
	m_batchPath = MATLAB_BASE_PATH;
	m_batchRepeat = _T("3");
	m_onlyNumNeigh = _T("5");
	m_restrictUserNearestNeigh = 4;
	m_neighbUsage = 0;
	m_percentKeysCompr = _T("80");
	m_useNeighAmplif = TRUE;
	m_useNeighDistrib = TRUE;
	m_removeMostComprBlack = FALSE;
	m_percentMostComprBlack = _T("15");
	m_removeMostComprWeighted = FALSE;
	m_randomComprDistrib = 0;
	m_blackRatioReceiveRange = _T("1");
	m_useFixedSeed = FALSE;
	m_randomSeed = _T("0");
	m_blackInGrid = FALSE;
	//}}AFX_DATA_INIT
}


void CBasicTab::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBasicTab)
	DDX_Radio(pDX, IDC_BLACK_COUNT_TYPE, m_blackCountType);
	DDX_Text(pDX, IDC_BLACK_RATIO_WHITE, m_blackRatioWhite);
	DDX_Text(pDX, IDC_BLACK_SENSORS_NUM, m_blackNumber);
	DDX_Text(pDX, IDC_PLANE_RADIUS, m_planeRadius);
	DDX_Radio(pDX, IDC_PLANE_TYPE, m_planeType);
	DDX_Text(pDX, IDC_PLANE_X, m_planeX);
	DDX_Text(pDX, IDC_PLANE_Y, m_planeY);
	DDX_Text(pDX, IDC_WHITE_LANDING_FAILURE, m_whiteLandFail);
	DDX_Text(pDX, IDC_WHITE_SENSORS_NUM, m_whiteNumber);
	DDX_Text(pDX, IDC_WHITE_TRANSM_RANGE, m_whiteTransmRange);
	DDX_Text(pDX, IDC_ONLY_NUM_NEIGH, m_onlyNumNeigh);
	DDX_Radio(pDX, IDC_NEIGHB_USAGE, m_neighbUsage);
	DDX_Text(pDX, IDC_PERCENT_KEYS_COMPROMISED, m_percentKeysCompr);
	DDX_Check(pDX, IDC_NEIGH_AMPLIF, m_useNeighAmplif);
	DDX_Check(pDX, IDC_NEIGH_DISTRIB, m_useNeighDistrib);
	DDX_Check(pDX, IDC_REMOVE_MOSTCOMPRBLACK, m_removeMostComprBlack);
	DDX_Text(pDX, IDC_PERCENT_MOSTCOMPRBLACK, m_percentMostComprBlack);
	DDX_Check(pDX, IDC_REMOVE_MOSTCOMPR_WEIGHTED, m_removeMostComprWeighted);
	DDX_Radio(pDX, IDC_RANDOM_COMPR_DISTRIB, m_randomComprDistrib);
	DDX_Text(pDX, IDC_BLACK_RATIO_RECEIVE_RANGE, m_blackRatioReceiveRange);
	DDX_Check(pDX, IDC_USE_FIXED_SEED, m_useFixedSeed);
	DDX_Text(pDX, IDC_RANDOM_SEED, m_randomSeed);
	DDX_Check(pDX, IDC_BLACK_IN_GRID, m_blackInGrid);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CBasicTab, CDialog)
	//{{AFX_MSG_MAP(CBasicTab)
	ON_BN_CLICKED(IDC_LOAD_CONFIG_SCRIPT, OnLoadConfigScript)
	ON_BN_CLICKED(IDC_SAVE_CONFIG_SCRIPT, OnSaveConfigScript)
	ON_BN_CLICKED(IDC_EXECUTE_CONFIG_SCRIPT, OnExecuteConfigScript)
	//}}AFX_MSG_MAP
    ON_BN_CLICKED(IDC_LOAD_WHITE_DEPLOY_PATTERN, &CBasicTab::OnBnClickedLoadWhiteDeployPattern)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBasicTab message handlers

int CBasicTab::GetDlgData(BASIC_INIT_DATA* pBasicSettings) {
    int     status = STAT_OK;

    UpdateData(TRUE);

    // PLANE
	pBasicSettings->planeRadius = (float) atof(m_planeRadius);
	pBasicSettings->planeType = m_planeType;
	pBasicSettings->planeX = (float) atof(m_planeX);
	pBasicSettings->planeY = (float) atof(m_planeY);

    // WHITE SENSORS
	pBasicSettings->whiteFailureRate = (float) atof(m_whiteLandFail);
	pBasicSettings->whiteNum = atoi(m_whiteNumber);
	pBasicSettings->whiteTransmRange = (float) atof(m_whiteTransmRange);

    // BLACK SENSORS
    pBasicSettings->blackCountType = m_blackCountType;
    pBasicSettings->blackRatioWhite = (float) atof(m_blackRatioWhite);
    switch (m_blackCountType) {
        case 0: {
        	pBasicSettings->blackNum = (DWORD) atoi(m_blackNumber);
            break;
        }
        case 1: {
        	if (atof(m_blackRatioWhite) != 0) {
                pBasicSettings->blackNum = (DWORD) (pBasicSettings->whiteNum * (atof(m_blackRatioWhite)));
            }
            else {
                //     
                LOG_INSERTSTRING("Black to white ratio cannot be 0\n.");
            }

            break;
        }
    }

	// BLACK HAVE SAME TRANSMISSION RANGE AS WHITE (- NOT IN USE)
    pBasicSettings->blackTransmRange = pBasicSettings->whiteTransmRange;
	// BLACK HAVE VARIABLE RECEIVE SENSITIVITY RATIO (ACCORDING TO WHITE TRANSMISSION RANGE)    
    pBasicSettings->blackRatioReceiveRange = (float) atof(m_blackRatioReceiveRange);

    // STYLE OF INITIAL KEY DISTRIBUTION
    switch (m_neighbUsage) {
        case 0: {
        	// ALL NEIGBOURS ARE USED
            pBasicSettings->useNeighFlags |= FLAG_USENEIGH_ALL;
            break;
        }
        case 1: {
        	// ONLY NEIGBOURS UP TO m_onlyNumNeigh ARE USED
            pBasicSettings->useNeighFlags |= FLAG_USENEIGH_ONLYXNEARST;
            break;
        }
    }
    pBasicSettings->useOnlyXNeigh = atoi(m_onlyNumNeigh);
    pBasicSettings->useNeighFlags |= (m_useNeighDistrib) ? FLAG_USENEIGH_DISTRIBUTION : 0;
    if (m_useNeighAmplif) {
        pBasicSettings->maintainCompromissedBy = TRUE;
        pBasicSettings->useNeighFlags |= FLAG_USENEIGH_AMPLIFICATION;
    }

    // DISTRIBUTION PROPERTIES
    switch (m_randomComprDistrib) {
        case 0: {
            // RANDOM DISTRIBUTION NOT USED
            break;
        }
        case 1: {
            pBasicSettings->distribFlags |= FLAG_DISTRIB_SKIPDISTRIBMARKRANDOMLY; 
            break;
        }
        case 2: {
            pBasicSettings->distribFlags |= FLAG_DISTRIB_MARKEQUALRANDOMLY; 
            break;
        }
    }

    pBasicSettings->distribKeysCompr = (float) atof(m_percentKeysCompr);
	pBasicSettings->distribFlags |= (m_removeMostComprBlack) ? FLAG_DISTRIB_REMOVEMOSTCOMPRBLACK : 0;
	pBasicSettings->distribFlags |= (m_removeMostComprWeighted) ? FLAG_DISTRIB_REMOVEMOSTCOMPRBLACKWEIGHTED : 0;
	pBasicSettings->distribFlags |= (m_blackInGrid) ? FLAG_DISTRIB_BLACKINGRID : 0;
	pBasicSettings->removeMostComprBlack = (float) atof(m_percentMostComprBlack);


    pBasicSettings->rndGen.randomSeed = atol(m_randomSeed);
    pBasicSettings->rndGen.useFixedSeed = m_useFixedSeed; 

    return status;
}

int CBasicTab::SetDlgData(BASIC_INIT_DATA* pBasicSettings) {
    int     status = STAT_OK;

    // PLANE
	m_planeRadius.Format("%.2f", pBasicSettings->planeRadius);
	m_planeType = pBasicSettings->planeType;
	m_planeX.Format("%.2f", pBasicSettings->planeX);
	m_planeY.Format("%.2f", pBasicSettings->planeY);

    // WHITE SENSORS
	m_whiteLandFail.Format("%.2f", pBasicSettings->whiteFailureRate);
	m_whiteNumber.Format("%d", pBasicSettings->whiteNum);
	m_whiteTransmRange.Format("%.2f", pBasicSettings->whiteTransmRange);

    // BLACK SENSORS
    m_blackCountType = pBasicSettings->blackCountType;
    m_blackNumber.Format("%d", pBasicSettings->blackNum); 
    m_blackRatioWhite.Format("%.2f", pBasicSettings->blackRatioWhite);

	// BLACK HAVE VARIABLE RECEIVE SENSITIVITY RATIO (ACCORDING TO WHITE TRANSMISSION RANGE)    
    m_blackRatioReceiveRange.Format("%.2f", pBasicSettings->blackRatioReceiveRange);

    // STYLE OF INITIAL KEY DISTRIBUTION
    if (pBasicSettings->useNeighFlags & FLAG_USENEIGH_ALL) m_neighbUsage = 0; 
    if (pBasicSettings->useNeighFlags & FLAG_USENEIGH_ONLYXNEARST) m_neighbUsage = 1; 

    m_onlyNumNeigh.Format("%d", pBasicSettings->useOnlyXNeigh);
    
    m_useNeighDistrib = (pBasicSettings->useNeighFlags & FLAG_USENEIGH_DISTRIBUTION) ? TRUE : FALSE;
    m_useNeighAmplif = (pBasicSettings->useNeighFlags & FLAG_USENEIGH_AMPLIFICATION) ? TRUE : FALSE;

    // DISTRIBUTION PROPERTIES
    m_randomComprDistrib = 0;
    if (pBasicSettings->distribFlags & FLAG_DISTRIB_SKIPDISTRIBMARKRANDOMLY) m_randomComprDistrib = 1; 
    if (pBasicSettings->distribFlags & FLAG_DISTRIB_MARKEQUALRANDOMLY) m_randomComprDistrib = 2; 

    m_percentKeysCompr.Format("%.2f", pBasicSettings->distribKeysCompr);
    m_removeMostComprBlack = (pBasicSettings->distribFlags & FLAG_DISTRIB_REMOVEMOSTCOMPRBLACK) ? TRUE : FALSE;
    m_removeMostComprWeighted = (pBasicSettings->distribFlags & FLAG_DISTRIB_REMOVEMOSTCOMPRBLACKWEIGHTED) ? TRUE : FALSE;
    m_blackInGrid = (pBasicSettings->distribFlags & FLAG_DISTRIB_BLACKINGRID) ? TRUE : FALSE;
    m_percentMostComprBlack.Format("%.2f", pBasicSettings->removeMostComprBlack);


    m_randomSeed.Format("%d", pBasicSettings->rndGen.randomSeed);
    m_useFixedSeed = pBasicSettings->rndGen.useFixedSeed; 

    UpdateData(FALSE);

    return status;
}

BOOL CBasicTab::OnInitDialog() 
{
	CDialog::OnInitDialog();
	

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}




void CBasicTab::OnLoadConfigScript() 
{
    int         status = STAT_OK;
    CFileDialog fileDialog(TRUE);
    CString     message;

    // BROWSE FOR PATH
    if (fileDialog.DoModal() == IDOK) {
        BASIC_INIT_DATA     basicSettings;

        if ((status = pSensorSim->m_sensorEnv.LoadConfigScript(fileDialog.GetPathName(), &basicSettings)) == STAT_OK) {
			message.Format("numDeploy: %d", basicSettings.numDeployments);
			LOG_INSERTSTRING(message);

            status = pSensorSim->SetBasicSettings(&basicSettings);
        }

        if (status != STAT_OK) {
            message.Format("Load config. script '%s' fail with %s(%d)", fileDialog.GetPathName(), ErrorToString(status), status);
            LOG_INSERTSTRING(message);
        } 
        else {
            message.Format("Load config. script '%s' OK", fileDialog.GetPathName());
            LOG_INSERTSTRING(message);
        }
    }

}

void CBasicTab::OnSaveConfigScript() 
{
    int         status = STAT_OK;
    CFileDialog fileDialog(FALSE);
    CString     message;

    // BROWSE FOR PATH
    if (fileDialog.DoModal() == IDOK) {
        // UPDATE ATTACK SCRIPT PATH
        BASIC_INIT_DATA     basicSettings;

        if ((status = pSensorSim->GetBasicSettings(&basicSettings)) == STAT_OK) {
            status = pSensorSim->m_sensorEnv.SaveConfigScript(fileDialog.GetPathName(), &basicSettings);
        }

        if (status != STAT_OK) { 
            message.Format("Save config. script '%s' fail with %s(%d)", fileDialog.GetPathName(), ErrorToString(status), status);
            LOG_INSERTSTRING(message);
        } 
        else {
            message.Format("Save config. script '%s' OK", fileDialog.GetPathName());
            LOG_INSERTSTRING(message);
        }
    }
	
}

void CBasicTab::OnExecuteConfigScript() 
{
    int         status = STAT_OK;
    CFileDialog fileDialog(TRUE);
    CString     message;

    // BROWSE FOR PATH
    if (fileDialog.DoModal() == IDOK) {
        // UPDATE ATTACK SCRIPT PATH
        BASIC_INIT_DATA     basicSettings;

        // LOAD ANF FILL GUI CONTROLS BY SCRIPT
        if ((status = pSensorSim->m_sensorEnv.LoadConfigScript(fileDialog.GetPathName(), &basicSettings)) == STAT_OK) {
            status = pSensorSim->SetBasicSettings(&basicSettings);
        }

        // EXECUTE TESTS FROM SCRIPT
        if (status == STAT_OK) {
            pSensorSim->m_sensorEnv.ExecuteTestsByScript(fileDialog.GetPathName());
        }

        if (status != STAT_OK) {
            message.Format("Execute config. script '%s' fail with %s(%d)", fileDialog.GetPathName(), ErrorToString(status), status);
            LOG_INSERTSTRING(message);
        } 
        else {
            message.Format("Execute config. script '%s' OK", fileDialog.GetPathName());
            LOG_INSERTSTRING(message);
        }
    }

}

void CBasicTab::OnBnClickedLoadWhiteDeployPattern()
{
    // TODO: Add your control notification handler code here
}
