#ifndef EACIRCUIT_H
#define EACIRCUIT_H

#include "md5.h"

/*

TODO: 
- Selector layer - only limited number of connections, but can connect all values from previous layer
- Prunning - at first pass prunne only functions, connections are prunned only for important functions
- Prediction method - all prediction methods are run at paralel (probably not good as circuit will be different for different methods)

/**/


class CEACircuit {
public:
    GA_CIRCUIT  m_gaCirc;    
    
public:
    CEACircuit();
    ~CEACircuit();
    static UINT RunGACircuit(LPVOID pParam);
	static int ExecuteForInput(CString textCircuit, CString textInput); 
	static int ExecuteFromText(CString textCircuit); 
	static int TestEAC(); 

protected:
    static void garng_initializer(GAGenome &g);
    static int garng_mutator(GAGenome &g, float pmut);
    static int garng_crossover(const GAGenome &p1, const GAGenome &p2,GAGenome *o1, GAGenome *o2);
    static float garng_fitness(GAGenome &g);
    
    static void GenerateTestData(int testVectGenerMethod, int numTestVectors, int numRounds = -1);
    static int ParseCircuit(CString textCircuit, DWORD* circuit, int* numLayers, int* intLayerSize, int* outLayerSize);
    static int ExecuteCircuit(GA_CIRCUIT* pCircuit, GA1DArrayGenome<DWORD>* pGenome, BYTE inputs[MAX_INPUTS], BYTE outputs[MAX_OUTPUTS]);  
    static int PrintCircuit(GAGenome &g, CString filePath = "", BYTE usePredictorMask[MAX_OUTPUTS] = NULL, BOOL bPruneCircuit = FALSE);  
    static int EvaluateCircuitSuccess(BYTE* outputs, BYTE* correctOutputs, BYTE* usePredictorsMask, int numOutputs, int predictType, int* pMatch, int* pTotalPredictCount, int* predictorMatch = NULL);
	static int GetFunctionLabel(DWORD functionID, DWORD connections, CString* pLabel);
	static int PruneCircuit(GAGenome &g, GAGenome &prunnedG);
	static int GetUsedNodes(GAGenome &g, BYTE usePredictorMask[MAX_OUTPUTS], BYTE displayNodes[MAX_GENOME_SIZE]);        
    static BOOL HasConnection(DWORD functionID, DWORD connectionMask, int fncSlot, int connectionOffset, int bit, BOOL* pbImplicitConnection);
    static BOOL HasImplicitConnection(DWORD functionID);
    static BOOL IsOperand(DWORD functionID, DWORD connectionMask, int fncSlot, int connectionOffset, int bit, CString* pOperand);
    static int GetNeutralValue(DWORD functionID, CString* pOperand);
	
    //
    //  MD5 HASH
    //    
protected:   
    static void MD5_Init(MD5_CTX *mdContext);
    static void MD5_Update(MD5_CTX *mdContext, unsigned char *inBuf, unsigned int inLen, int numRounds = 64);
    static void MD5_Final(MD5_CTX *mdContext, int numRounds = 64);
    static void MD5_Transform (UINT4 *buf, UINT4 *in, int numRounds);
    
};


class CSHA1
{

	public:

		CSHA1();
		virtual ~CSHA1();

		/*
		 *	Re-initialize the class
		 */
		void Reset();

		/*
		 *	Returns the message digest
		 */
		bool Result(BYTE *message_digest_array);

		/*
		 *	Provide input to SHA1
		 */
		void Input(	const unsigned char	*message_array,
					unsigned			length);
		void Input(	const char	*message_array,
					unsigned	length);
		void Input(unsigned char message_element);
		void Input(char message_element);
		CSHA1& operator<<(const char *message_array);
		CSHA1& operator<<(const unsigned char *message_array);
		CSHA1& operator<<(const char message_element);
		CSHA1& operator<<(const unsigned char message_element);

	private:

		/*
		 *	Process the next 512 bits of the message
		 */
		void ProcessMessageBlock();

		/*
		 *	Pads the current message block to 512 bits
		 */
		void PadMessage();

		/*
		 *	Performs a circular left shift operation
		 */
		inline unsigned CircularShift(int bits, unsigned word);

		unsigned H[5];						// Message digest buffers

		unsigned Length_Low;				// Message length in bits
		unsigned Length_High;				// Message length in bits

		unsigned char Message_Block[64];	// 512-bit message blocks
		int Message_Block_Index;			// Index into message block array

		bool Computed;						// Is the digest computed?
		bool Corrupted;						// Is the message digest corruped?
};

#endif