// KeyEstablishTab.cpp : implementation file
// jen takova zmena

#include "stdafx.h"
#include "SensorSim.h"
#include "KeyEstablishTab.h"
#include "SensorSimDlg.h"
#include "TestQueueMngDlg.h"
#include "SavedCharsDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CSensorSimDlg*  pSensorSim;

/////////////////////////////////////////////////////////////////////////////
// CKeyEstablishTab dialog


CKeyEstablishTab::CKeyEstablishTab(CWnd* pParent /*=NULL*/)
	: CDialog(CKeyEstablishTab::IDD, pParent) 
{
	//{{AFX_DATA_INIT(CKeyEstablishTab)
	m_batchPath = MATLAB_BASE_PATH;
	m_batchRepeat = _T("1");
	m_rangeFrom = _T("0.0");
	m_rangeStep = _T("0.1");
	m_rangeTo = _T("1");
	m_useKeyInfect = FALSE;
	m_bindBlackRatio = FALSE;
	m_useMMD = FALSE;
	m_testQueueing = FALSE;
	m_note = _T("");
	m_useFOE = FALSE;
	m_useNLeash = TRUE;
//    m_whatToSaveFlags = FLAG_WHATSAVE_AMPLIFICATIONIMPACT | FLAG_WHATSAVE_COMPROMISSEDBYDISTANCE | FLAG_WHATSAVE_KEYSDISTRIBUTION | FLAG_WHATSAVE_SENSORPOSITION | FLAG_WHATSAVE_COMPRATIOALLMETHODS | FLAG_WHATSAVE_COMPROMISSEDBYNODEDENSITTY;
    m_whatToSaveFlags = FLAG_WHATSAVE_NEIGHBLEASH;
	m_optimalPoolSize = FALSE;
	//}}AFX_DATA_INIT
    m_testQueueExit = FALSE;
}


void CKeyEstablishTab::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CKeyEstablishTab)
    DDX_Text(pDX, IDC_BATCH_PATH, m_batchPath);
    DDX_Text(pDX, IDC_BATCH_REPEAT, m_batchRepeat);
    DDX_Text(pDX, IDC_RANGE_FROM, m_rangeFrom);
    DDX_Text(pDX, IDC_RANGE_STEP, m_rangeStep);
    DDX_Text(pDX, IDC_RANGE_TO, m_rangeTo);
    DDX_Check(pDX, IDC_USE_KEYINFECT, m_useKeyInfect);
    DDX_Check(pDX, IDC_BIND_BLACK_RATIO, m_bindBlackRatio);
    DDX_Check(pDX, IDC_USE_MMD, m_useMMD);
    DDX_Check(pDX, IDC_TEST_QUEUEING, m_testQueueing);
    DDX_Text(pDX, IDC_NOTE, m_note);
    DDX_Check(pDX, IDC_USE_FOE, m_useFOE);
    DDX_Check(pDX, IDC_OPTIM_POOL_SIZE, m_optimalPoolSize);
    //}}AFX_DATA_MAP
    DDX_Check(pDX, IDC_USE_NLEASH, m_useNLeash);
}


BEGIN_MESSAGE_MAP(CKeyEstablishTab, CDialog)
	//{{AFX_MSG_MAP(CKeyEstablishTab)
	ON_BN_CLICKED(IDC_BROWSE_BATCH_PATH, OnBrowseBatchPath)
	ON_BN_CLICKED(IDC_EXECUTE_FIXEDTEST, OnExecuteFixedtest)
	ON_BN_CLICKED(IDC_EXECUTE_VARIABLERANGETESTS, OnExecuteVariablerangetests)
	ON_BN_CLICKED(IDC_STOP_CURRENT_OPERATIONS, OnStopCurrentOperations)
	ON_BN_CLICKED(IDC_EXECUTE_VARIABLERATIOTESTS, OnExecuteVariableratiotests)
	ON_BN_CLICKED(IDC_EXECUTE_VARIABLERECEIVERATIOTESTS, OnExecuteVariablereceiveratiotests)
	ON_BN_CLICKED(IDC_EXECUTE_VARIABLEBSNUMBERTEST, OnExecuteVariabletotalbscounttest)
	ON_BN_CLICKED(IDC_EXECUTE_VARIABLEACTIVEBSCOUNTTEST, OnExecuteVariableactivebscounttest)
	ON_BN_CLICKED(IDC_BATCH_QUEUE_MNG, OnBatchQueueMng)
	ON_BN_CLICKED(IDC_SAVED_CHARACTERISTICS, OnSavedCharacteristics)
	ON_BN_CLICKED(IDC_VARIABLE_COMPR_NODES, OnExecuteVariableComprNodesTest)
	ON_BN_CLICKED(IDC_VARIABLE_WHITE_NODES_COUNT, OnVariableWhiteNodesCount)
	ON_BN_CLICKED(IDC_EXECUTE_VARIABLETOTALBSCOUNTTEST, OnExecuteVariabletotalbscounttest)
	ON_BN_CLICKED(IDC_VARIABLE_POOLSIZE, OnVariablePoolsize)
    ON_BN_CLICKED(IDC_VARIABLE_DROPCHECK_PROB, OnBnClickedVariableDropcheckProb)
    ON_BN_CLICKED(IDC_VARIABLE_COMPRNODESFRACT_PROB, OnBnClickedVariableComprnodesfractProb)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CKeyEstablishTab message handlers

void CKeyEstablishTab::OnBrowseBatchPath() 
{
    int         status = STAT_OK;
    CFileDialog fileDialog(TRUE);
    CString     fileName;

    UpdateData(TRUE);

    if (fileDialog.DoModal() == IDOK) {
        fileName = fileDialog.GetPathName();
        fileName = fileName.Left(fileName.GetLength() - fileDialog.GetFileName().GetLength());
        m_batchPath = fileName;
    }

	UpdateData(FALSE);
}

int CKeyEstablishTab::GetDlgData(BASIC_INIT_DATA* pBasicSettings) {
    int status = STAT_OK;
    
    UpdateData(TRUE);

    // VALUES STORED FROM CSavedCharsDlg
    pBasicSettings->batchTest.whatSaveFlags = m_whatToSaveFlags;

    if (pBasicSettings->batchTest.whatSaveFlags & FLAG_WHATSAVE_COMPROMISSEDBYSTATS) {
        // MAINTAING OF COMPROMISSED_BY LIST MUST BE ALLOWED
        pBasicSettings->maintainCompromissedBy = TRUE;
    }

    if (m_useKeyInfect) pBasicSettings->batchTest.methodsFlags |= FLAG_METHOD_USEKEYINFECT;
    if (m_useMMD) {
        pBasicSettings->batchTest.methodsFlags |= FLAG_METHOD_USEMULTIPATHMESSAGEDELIVERY;
        pBasicSettings->batchTest.whatSaveFlags |= FLAG_WHATSAVE_MMD;
    }
    if (m_useFOE) {
        pBasicSettings->batchTest.methodsFlags |= FLAG_METHOD_USEFORWARDONIONENCRYPTION;
        pBasicSettings->batchTest.whatSaveFlags |= FLAG_WHATSAVE_FOE;
    }
    if (m_useNLeash) {
        pBasicSettings->batchTest.methodsFlags |= FLAG_METHOD_USENEIGHBLEASH;
        pBasicSettings->batchTest.whatSaveFlags |= FLAG_WHATSAVE_NEIGHBLEASH;
    }
    
    if (m_bindBlackRatio) pBasicSettings->distribFlags |= FLAG_DISTRIB_BINDBLACKCOUNTRECEIVERATIO;
    if (m_optimalPoolSize) pBasicSettings->batchTest.methodsFlags |= FLAG_METHOD_USEOPTIMALPOOLSIZE;
 
 
    pBasicSettings->batchTest.batchRepeat = atoi(m_batchRepeat);
    pBasicSettings->batchTest.rangeFrom = (float) atof(m_rangeFrom);
    pBasicSettings->batchTest.rangeTo = (float) atof(m_rangeTo);
    pBasicSettings->batchTest.rangeStep = (float) atof(m_rangeStep);
    pBasicSettings->batchTest.outputPath = m_batchPath;
    pBasicSettings->batchTest.note = m_note;

    return status;
}

int CKeyEstablishTab::SetDlgData(BASIC_INIT_DATA* pBasicSettings) {
    int status = STAT_OK;

    // VALUES DISPLAYED IN CSavedCharsDlg
    m_whatToSaveFlags = pBasicSettings->batchTest.whatSaveFlags;

    m_useKeyInfect = (pBasicSettings->batchTest.methodsFlags & FLAG_METHOD_USEKEYINFECT) ? TRUE : FALSE;
    m_useMMD = (pBasicSettings->batchTest.methodsFlags & FLAG_METHOD_USEMULTIPATHMESSAGEDELIVERY) ? TRUE : FALSE;
    m_useFOE = (pBasicSettings->batchTest.methodsFlags & FLAG_METHOD_USEFORWARDONIONENCRYPTION) ? TRUE : FALSE;
    m_useNLeash = (pBasicSettings->batchTest.methodsFlags & FLAG_METHOD_USENEIGHBLEASH) ? TRUE : FALSE;
    
    
    m_bindBlackRatio = (pBasicSettings->distribFlags & FLAG_DISTRIB_BINDBLACKCOUNTRECEIVERATIO) ? TRUE : FALSE;
    m_optimalPoolSize = (pBasicSettings->batchTest.methodsFlags & FLAG_METHOD_USEOPTIMALPOOLSIZE) ? TRUE : FALSE;
    
    m_batchRepeat.Format("%d", pBasicSettings->batchTest.batchRepeat);
    m_rangeFrom.Format("%.2f", pBasicSettings->batchTest.rangeFrom);
    m_rangeTo.Format("%.2f", pBasicSettings->batchTest.rangeTo);
    m_rangeStep.Format("%.2f", pBasicSettings->batchTest.rangeStep);
    m_batchPath = pBasicSettings->batchTest.outputPath;
    m_note = pBasicSettings->batchTest.note;


    UpdateData(FALSE);

    return status;
}

int CKeyEstablishTab::ExecuteTestThread(DWORD testType) {
    int                     status = STAT_OK;
    BASIC_INIT_DATA         basicSettings;
    
    UpdateData(TRUE);

    if ((status = pSensorSim->GetBasicSettings(&basicSettings)) == STAT_OK) {
        status = ExecuteTestThread(&basicSettings, testType);
    }	

    return status;
}

int CKeyEstablishTab::ExecuteTestThread(BASIC_INIT_DATA* pBasicSettings, DWORD testType) {
    int                     status = STAT_OK;
    TEST_THREAD_PARAM*      pThreadParam = new TEST_THREAD_PARAM;
    CString                 cfgScriptPath;
    AFX_THREADPROC          threadProc = NULL;
    
    // SET BASIC SETTINGS
    pThreadParam->basicSettings = *pBasicSettings;
    
    LOG_INSERTSEPARATOR;
    
    // INIT RANDOM GENERATOR
    pSensorSim->m_sensorEnv.InitRandomGenerator(&(pThreadParam->basicSettings));
    
    // SET SEED FIXED FOR FUTURE RE-EXECUTIONS OF SAME TEST
    pThreadParam->basicSettings.rndGen.useFixedSeed = TRUE;

    // SET TEST TYPE
    pThreadParam->basicSettings.testType = testType;

    // SAVE CONFIGURATION SCRIPT
    cfgScriptPath.Format("%s\\!cfgScript.ini", pThreadParam->basicSettings.batchTest.outputPath);
    status = pSensorSim->m_sensorEnv.SaveConfigScript(cfgScriptPath, &(pThreadParam->basicSettings));

    // TODO: CHECK FOR ALREADY RUNNING THREADS AND END THEM CORRETLY
    // INTERRUPT PREVIOUS THREAD
    if (pSensorSim->m_pActualThread != NULL) TerminateThread(pSensorSim->m_pActualThread->m_hThread, 0);

    // RESET SOFT FLAG FOR STOPPING CURRENT FLAG
    pSensorSim->m_stopThreadSoft = FALSE;

    switch (testType) {
        case FLAG_TESTTYPE_FIXEDTRANSMRANGE: threadProc = CSensorEnv::ExecuteFixedTest; break;
        case FLAG_TESTTYPE_VARIABLETRANSMRANGE: threadProc = CSensorEnv::ExecuteVariableRangeTest; break;
        case FLAG_TESTTYPE_VARIABLEBWCOUNTRATIO: threadProc = CSensorEnv::ExecuteVariableBWRatioTest; break;
        case FLAG_TESTTYPE_VARIABLEBLACKRECEIVESENSITIVITY: threadProc = CSensorEnv::ExecuteVariableBlackReceiveRatioTest; break;
        case FLAG_TESTTYPE_VARIABLETOTALBASESTATIONCOUNT: threadProc = CSensorEnv::ExecuteVariableTotalBSNumberTest; break;
        case FLAG_TESTTYPE_VARIABLEACTIVEBASESTATIONCOUNT: threadProc = CSensorEnv::ExecuteVariableActiveBSNumberTest; break;
        case FLAG_TESTTYPE_VARIABLECOMPRNODESCOUNT: threadProc = CSensorEnv::ExecuteVariableComprNodesTest; break;
        case FLAG_TESTTYPE_VARIABLEWHITENODESCOUNT: threadProc = CSensorEnv::ExecuteVariableWhiteNodesTest; break;
        case FLAG_TESTTYPE_VARIABLEDROPCHECKPROB: threadProc = CSensorEnv::ExecuteVariableDropCheckProbTest; break; 
        case FLAG_TESTTYPE_VARIABLECOMPRNODES: threadProc = CSensorEnv::ExecuteVariableComprFractTest; break; 
        default: status = STAT_USERDATA_BAD;
    }

    if (threadProc != NULL) {
        // RUN WORKER THREAD FOR EXECUTION
        pSensorSim->m_pActualThread = AfxBeginThread(threadProc, pThreadParam, THREAD_PRIORITY_LOWEST); 
    }

    return status;
}

void CKeyEstablishTab::OnStopCurrentOperations() 
{
    // SET SOFT FLAG FOR STOPPING CURRENT FLAG
    pSensorSim->m_stopThreadSoft = TRUE;

/*
// SIMPLE LOAD AND SAVE TEST
    BASIC_INIT_DATA     basicSettings1;
    BASIC_INIT_DATA     basicSettings2;
    BASIC_INIT_DATA     basicSettings3;

    pSensorSim->GetBasicSettings(&basicSettings1);
    pSensorSim->SaveConfigScript("C:\\cfg.ini", &basicSettings1);
    pSensorSim->LoadConfigScript("C:\\cfg.ini", &basicSettings2);
    
    ASSERT(memcmp(&basicSettings1, &basicSettings2, sizeof(BASIC_INIT_DATA)) == 0);

    pSensorSim->SetBasicSettings(&basicSettings2);
    pSensorSim->GetBasicSettings(&basicSettings3);

    ASSERT(memcmp(&basicSettings2, &basicSettings3, sizeof(BASIC_INIT_DATA)) == 0);
/**/
}


void CKeyEstablishTab::OnExecuteFixedtest() 
{
    //
    //  RATIO OF COMPROMISSED KEYS BY DISTANCE FROM CENTRAL NODE UP TO MAX. RANGE 
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_FIXEDTRANSMRANGE);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_FIXEDTRANSMRANGE);
    }
}

void CKeyEstablishTab::OnExecuteVariablerangetests() 
{
    //
    //  SECURED KEYS TO NUMBER OF NEIGHBOURS DEPENDENCY
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLETRANSMRANGE);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLETRANSMRANGE);
    }
}


void CKeyEstablishTab::OnExecuteVariableratiotests() 
{
    //
    //  SECURED KEYS TO NUMBER OF NEIGHBOURS DEPENDENCY
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLEBWCOUNTRATIO);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLEBWCOUNTRATIO);
    }
}

void CKeyEstablishTab::OnExecuteVariablereceiveratiotests() 
{
    //
    //  SECURED KEYS TO BLACK SENSITIVITY DEPENDENCY
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLEBLACKRECEIVESENSITIVITY);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLEBLACKRECEIVESENSITIVITY);
    }
}

void CKeyEstablishTab::OnExecuteVariabletotalbscounttest() 
{
    //
    //  SECURED MESSAGES TO TOTAL COUNT ASE STATIONS WITH 1 ACTIVE BS
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLETOTALBASESTATIONCOUNT);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLETOTALBASESTATIONCOUNT);
    }
}

void CKeyEstablishTab::OnExecuteVariableactivebscounttest() 
{
    //
    //  SECURED MESSAGES TO TOTAL COUNT ASE STATIONS WITH 1 ACTIVE BS
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLEACTIVEBASESTATIONCOUNT);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLEACTIVEBASESTATIONCOUNT);
    }
}


void CKeyEstablishTab::OnExecuteVariableComprNodesTest() 
{
    //
    //  IMPACT OF VARIABLE NUMBER OF COMPROMISSED NODES  
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLECOMPRNODESCOUNT);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLECOMPRNODESCOUNT);
    }
}

void CKeyEstablishTab::OnBatchQueueMng() 
{
    int                 status = STAT_OK;
    CTestQueueMngDlg	mngDlg;

    if (mngDlg.DoModal() == IDOK) {
        switch (mngDlg.m_actionType) {
            case TESTQUEUE_RUN: {
                // RUN ALL TESTS
                TestQueue_Run();
                break;
            }  
            case TESTQUEUE_CLEAR: {
                // CLEAR QUEUE
                TestQueue_Clear();
                break;
            }
        }
    }
}


int CKeyEstablishTab::TestQueue_Add(DWORD testType) {
    int              status = STAT_OK;
    BASIC_INIT_DATA* pBasicSettings;
        


    pBasicSettings = new BASIC_INIT_DATA;

    // GET TEST SETTINGS
    if ((status = pSensorSim->GetBasicSettings(pBasicSettings)) == STAT_OK) {
        // SET TEST TYPE
        pBasicSettings->testType = testType;

        status = TestQueue_Add(pBasicSettings);
    }
    
    return status;
}

int CKeyEstablishTab::TestQueue_Add(BASIC_INIT_DATA* pBasicSettings) {
    int              status = STAT_OK;
    CString          message;

    if (pBasicSettings != NULL) {
        // SET APP VERSION IF NOT SET BEFORE
        if (pBasicSettings->simulSWVersion == "") pSensorSim->GetBasicSettingsHeader(pBasicSettings);

        // ADD TO QUEUE
        m_testQueue.push_back(pBasicSettings);

        message.Format("Test number %d added into queue ('%s')", m_testQueue.size(),  pBasicSettings->batchTest.outputPath);
        LOG_INSERTSTRING(message);
    }

    return status;
}


UINT TestQueueRun(LPVOID pParam) {
    UINT                status = STAT_OK;
    TEST_THREAD_PARAM*  pThreadParam = (TEST_THREAD_PARAM*) pParam;
    bsl::iterator       iter;
    bsl*                pTestQueue = (bsl*) pThreadParam->pTestQueue;
    BASIC_INIT_DATA*    pBasicSettings;
    CString             cfgScriptPath;
    CString             message;
    
    // EXECUTE TESTS IN QUEUE
    for (iter = pTestQueue->begin(); iter != pTestQueue->end(); iter++) {
        pBasicSettings = *iter;
        
        message.Format("Next test from queue ('%s')", pBasicSettings->batchTest.outputPath);
        LOG_INSERTSTRING(message);
        LOG_INSERTSEPARATOR;

        // INIT RANDOM GENERATOR
        pSensorSim->m_sensorEnv.InitRandomGenerator(pBasicSettings);
    
        // SET SEED FIXED FOR FUTURE RE-EXECUTIONS OF SAME TEST
        pBasicSettings->rndGen.useFixedSeed = TRUE;

        // SAVE CONFIGURATION SCRIPT
        cfgScriptPath.Format("%s\\!cfgScript.ini", pBasicSettings->batchTest.outputPath);
        status = pSensorSim->m_sensorEnv.SaveConfigScript(cfgScriptPath, pBasicSettings);

        status = pSensorSim->m_sensorEnv.ExecuteTestsBySettings(pBasicSettings);
    }

    // CLEAR LOCAL TEST QUEUE
    for (iter = pTestQueue->begin(); iter != pTestQueue->end(); iter++) {
        pBasicSettings = *iter;
        if (pBasicSettings != NULL) delete pBasicSettings;
    }
    pTestQueue->clear();

    // EXIT APP IF REQUIRED
    if (pThreadParam->bExitApp) pSensorSim->PostMessage(WM_CLOSE);

    delete (bsl*) pThreadParam->pTestQueue;
    delete pThreadParam;

    LOG_INSERTSTRING("TestQueueRun finished");



    return status;
}

int CKeyEstablishTab::TestQueue_Run() {
    int                 status = STAT_OK;
    TEST_THREAD_PARAM*  pThreadParam = new TEST_THREAD_PARAM;
    bsl*                pLocalQueue;
    bsl::iterator       iter;
    BASIC_INIT_DATA*    pBasicSettings;
    BASIC_INIT_DATA*    pBasicSettings2;

    pLocalQueue = new bsl;

    // CREATE LOCAL COPY OF TEST QUEUE
    for (iter = m_testQueue.begin(); iter != m_testQueue.end(); iter++) {
        pBasicSettings = *iter;
        
        pBasicSettings2 = new BASIC_INIT_DATA;
        *pBasicSettings2 = *pBasicSettings;

        pLocalQueue->push_back(pBasicSettings2);
    }

    // SET LOCAL QUEUE AS THREAD PARAM
    pThreadParam->pTestQueue = pLocalQueue;
    pThreadParam->bExitApp = m_testQueueExit;

    // TODO: CHECK FOR ALREADY RUNNING THREADS AND END THEM CORRETLY
    // INTERRUPT PREVIOUS THREAD
    if (pSensorSim->m_pActualThread != NULL) TerminateThread(pSensorSim->m_pActualThread->m_hThread, 0);

    // RESET SOFT FLAG FOR STOPPING CURRENT FLAG
    pSensorSim->m_stopThreadSoft = FALSE;
    // RUN WORKER THREAD FOR QUEUE TESTS
    pSensorSim->m_pActualThread = AfxBeginThread(TestQueueRun, pThreadParam, THREAD_PRIORITY_LOWEST); 

    // CLEAR ORIGINAL TEST QUEUE
    TestQueue_Clear();

    return status;
}

int CKeyEstablishTab::TestQueue_Clear() {
    int                 status = STAT_OK;
    bsl::iterator       iter;
    BASIC_INIT_DATA*    pBasicSettings;
    
    for (iter = m_testQueue.begin(); iter != m_testQueue.end(); iter++) {
        pBasicSettings = *iter;

        if (pBasicSettings != NULL) delete pBasicSettings;
    }
    
    m_testQueue.clear();

    LOG_INSERTSTRING("Test queue cleared.");

    return status;
}

int CKeyEstablishTab::TestQueue_ExitAfterTests(BOOL bExit) {
    m_testQueueExit = bExit;
    return STAT_OK;
}

void CKeyEstablishTab::OnSavedCharacteristics() 
{
    CSavedCharsDlg  charsDlg(this); 
    
    charsDlg.m_whatToSaveFlags = m_whatToSaveFlags;
    if (charsDlg.DoModal() == IDOK) {
        m_whatToSaveFlags = charsDlg.m_whatToSaveFlags;
    }
}


void CKeyEstablishTab::OnVariableWhiteNodesCount() 
{
    //
    //  CHANGING NUMBER OF WHITE NODES
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLEWHITENODESCOUNT);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLEWHITENODESCOUNT);
    }
}

void CKeyEstablishTab::OnVariablePoolsize() 
{
    //
    //  CHANGING NUMBER OF KEYS IN POOL
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLEKEYPOOLSIZE);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLEKEYPOOLSIZE);
    }
}

void CKeyEstablishTab::OnBnClickedVariableDropcheckProb()
{
    //
    //  CHANGING PROBABILITY OF CHECK FROM MESSAGE DROPPING
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLEDROPCHECKPROB);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLEDROPCHECKPROB);
    }
}

void CKeyEstablishTab::OnBnClickedVariableComprnodesfractProb()
{
    //
    //  CHANGING FRACTION OF COMPROMISSED NODES
    //
    UpdateData(TRUE);

    if (m_testQueueing) {
        // PUT TEST INTO QUEUE
        TestQueue_Add(FLAG_TESTTYPE_VARIABLECOMPRNODES);
    }
    else {
        // EXECUTE TEST DIRECTLY
        ExecuteTestThread(FLAG_TESTTYPE_VARIABLECOMPRNODES);
    }
}
