// KeyPredistribTab.cpp : implementation file
//

#include "stdafx.h"
#include "SensorSim.h"
#include "KeyPredistribTab.h"
#include "SensorSimDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CSensorSimDlg*  pSensorSim;

/////////////////////////////////////////////////////////////////////////////
// CKeyPredistribTab dialog


CKeyPredistribTab::CKeyPredistribTab(CWnd* pParent /*=NULL*/)
	: CDialog(CKeyPredistribTab::IDD, pParent)
    , m_assumeSecureLinks(FALSE)
    , m_ringBitMask(false)
    , m_nodesInGroup(_T("40"))
    , m_ringAsBitMask(FALSE)
    , m_minKeysCount(_T("1"))
    , m_numPickedNodes(_T("100"))
{
	//{{AFX_DATA_INIT(CKeyPredistribTab)
	m_intersectSize = _T("1");
	m_keyPoolSize = _T("132200");
	m_keyRingSize = _T("200");
	m_egScheme = TRUE;
	m_usePullAmplif = FALSE;
	m_usePushAmplif = FALSE;
	m_amplifRepeat = _T("1");
	m_maxComprNodes = _T("500");
	m_useLocalFOEBoost = TRUE;
	m_foeBoostCount = _T("0");
	m_foeBoostAllDirectNeigh = TRUE;
	m_notUseFOE = FALSE;
	//}}AFX_DATA_INIT
}


void CKeyPredistribTab::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CKeyPredistribTab)
    DDX_Text(pDX, IDC_INTERSECT_SIZE, m_intersectSize);
    DDX_Text(pDX, IDC_KEY_POOL_SIZE, m_keyPoolSize);
    DDX_Text(pDX, IDC_KEY_RING_SIZE, m_keyRingSize);
    DDX_Check(pDX, IDC_EG_SCHEME, m_egScheme);
    DDX_Check(pDX, IDC_USE_PULL_AMPLIF, m_usePullAmplif);
    DDX_Check(pDX, IDC_USE_PUSH_AMPLIF, m_usePushAmplif);
    DDX_Text(pDX, IDC_AMPLIF_REPEAT, m_amplifRepeat);
    DDX_Text(pDX, IDC_MAX_COMPR_NODES, m_maxComprNodes);
    DDX_Check(pDX, IDC_USE_LOCAL_FOE_BOOST, m_useLocalFOEBoost);
    DDX_Text(pDX, IDC_FOE_BOOST_COUNT, m_foeBoostCount);
    DDX_Check(pDX, IDC_FOE_USE_ALL_DIRECTNEIGH, m_foeBoostAllDirectNeigh);
    DDX_Check(pDX, IDC_NOT_USE_FOE, m_notUseFOE);
    //}}AFX_DATA_MAP
    DDX_Text(pDX, IDC_NODESINGROUP, m_nodesInGroup);
    DDX_Check(pDX, IDC_RINGBITMASK, m_ringAsBitMask);
    DDX_Text(pDX, IDC_MINKEYSCOUNT, m_minKeysCount);
}


BEGIN_MESSAGE_MAP(CKeyPredistribTab, CDialog)
	//{{AFX_MSG_MAP(CKeyPredistribTab)
	ON_BN_CLICKED(IDC_DEPLOY_SENSORS, OnDeploySensors)
	ON_BN_CLICKED(IDC_RECALCULATE, OnRecalculate)
	ON_BN_CLICKED(IDC_TEST_FOE, OnTestFoe)
	ON_BN_CLICKED(IDC_CALC_SHARE_PROB, OnCalcShareProb)
	ON_BN_CLICKED(IDC_PROB_AUTH, OnProbAuthentication)
	//}}AFX_MSG_MAP
    ON_BN_CLICKED(IDC_GAS_NODEPICK, &CKeyPredistribTab::OnBnClickedGasNodepick)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CKeyPredistribTab message handlers

void CKeyPredistribTab::OnDeploySensors() 
{
    int     status = STAT_OK;
    BASIC_INIT_DATA     basicSettings;
    
    UpdateData(TRUE);
 
    LOG_INSERTSEPARATOR;
    
    // LOAD INSERTED DATA
    if ((status = pSensorSim->GetBasicSettings(&basicSettings)) == STAT_OK) {
        // INIT RANDOM GENERATOR
        pSensorSim->m_sensorEnv.InitRandomGenerator(&basicSettings);

        if (m_egScheme) basicSettings.distribFlags |= FLAG_DISTRIB_USEEGSCHEME;

        // DEPLOY SENSORS
        pSensorSim->m_sensorEnv.DeploySensors(&basicSettings);
        
        // COMPUTE STATISTICS
        ComputeStatistics(&basicSettings);
    }	
}

void CKeyPredistribTab::OnRecalculate() 
{
    int     status = STAT_OK;
    BASIC_INIT_DATA     basicSettings;
    
    UpdateData(TRUE);
 
    LOG_INSERTSEPARATOR;
    
    // LOAD INSERTED DATA
    if ((status = pSensorSim->GetBasicSettings(&basicSettings)) == STAT_OK) {
        //TODO: SAVE DATA

        if (m_egScheme) basicSettings.distribFlags |= FLAG_DISTRIB_USEEGSCHEME;

        // RE-COMPUTE STATISTICS
        pSensorSim->m_sensorEnv.ResetNetwork();
        ComputeStatistics(&basicSettings);
    }	
}

int CKeyPredistribTab::ComputeStatistics(BASIC_INIT_DATA* pBasicSettings) {
    int     status = STAT_OK;
    DWORD   secure = 0;
    DWORD   missing = 0;
    DWORD   max;
    DWORD   min;
    float   avg;
    float   avgWhiteNeighBeforeBlack = 0;
    CString message;
    CString message2;
    CString fileName;
    #define MAX_NUM_NEIGH   1000
    float distWhite[MAX_NUM_NEIGH];
    memset(distWhite, 0, MAX_NUM_NEIGH * sizeof(float));


    // COMPUTE STATISTICS

    pSensorSim->m_sensorEnv.CalcWhiteNeighStats(&max, &min, &avg, &avgWhiteNeighBeforeBlack, distWhite);
    message.Format("White neighbours -- max: %d | min: %d | avg: %0.1f | avgWhiteBeforeBlack: %0.3f", max, min, avg, avgWhiteNeighBeforeBlack);
    LOG_INSERTSTRING(message);

    message = "";
    for (int i = 1; i < (int) avg; i++) {
        message2.Format("[%d:%0.2f] ", i, distWhite[i]);            
        message += message2;
    }
    LOG_INSERTSTRING(message);

    pSensorSim->m_sensorEnv.CalcBlackNeighStats(&max, &min, &avg);
    message.Format("Black neighbours -- max: %d | min: %d | avg: %0.1f", max, min, avg);
    LOG_INSERTSTRING(message);



	if (m_egScheme) {
        secure = missing = 0;
        pSensorSim->m_sensorEnv.ClearLinkKeysStatus();
        pSensorSim->m_sensorEnv.EGScheme(pBasicSettings, &secure, &missing);
        message.Format("EG scheme random predistribution -- total:%d | secure: %.2f%s | missing: %.2f%s", secure + missing, (secure / (float) (secure + missing)) * 100, "%", (missing / (float) (secure + missing)) * 100, "%");
        LOG_INSERTSTRING(message);

        if (m_usePullAmplif) {
            secure = missing = 0;
            pSensorSim->m_sensorEnv.ClearLinkKeysStatus();
            pSensorSim->m_sensorEnv.PullAmplifEGScheme(pBasicSettings, &secure, &missing, 1);
            message.Format("Pull amplif (1x) -- total:%d | secure: %.2f%s | missing: %.2f%s", secure + missing, (secure / (float) (secure + missing)) * 100, "%", (missing / (float) (secure + missing)) * 100, "%");
            LOG_INSERTSTRING(message);

            pSensorSim->m_sensorEnv.ClearLinkKeysStatus();
            if (atoi(m_amplifRepeat) > 1) {
                pSensorSim->m_sensorEnv.PullAmplifEGScheme(pBasicSettings, &secure, &missing, atoi(m_amplifRepeat));
                message.Format("Pull amplif (%dx) -- total:%d | secure: %.2f%s | missing: %.2f%s", atoi(m_amplifRepeat), secure + missing, (secure / (float) (secure + missing)) * 100, "%", (missing / (float) (secure + missing)) * 100, "%");
                LOG_INSERTSTRING(message);
            }
        }
        if (m_usePushAmplif) {
            secure = missing = 0;
            pSensorSim->m_sensorEnv.ClearLinkKeysStatus();
            pSensorSim->m_sensorEnv.PushAmplifEGScheme(pBasicSettings, &secure, &missing, atoi(m_amplifRepeat));
            message.Format("Push amplif (1x) -- total:%d | secure: %.2f%s | missing: %.2f%s", secure + missing, (secure / (float) (secure + missing)) * 100, "%", (missing / (float) (secure + missing)) * 100, "%");
            LOG_INSERTSTRING(message);
            
            pSensorSim->m_sensorEnv.ClearLinkKeysStatus();
            if (atoi(m_amplifRepeat) > 1) {
                pSensorSim->m_sensorEnv.PushAmplifEGScheme(pBasicSettings, &secure, &missing, atoi(m_amplifRepeat));
                message.Format("Push amplif (%dx) -- total:%d | secure: %.2f%s | missing: %.2f%s", atoi(m_amplifRepeat), secure + missing, (secure / (float) (secure + missing)) * 100, "%", (missing / (float) (secure + missing)) * 100, "%");
                LOG_INSERTSTRING(message);
            }
        }
    }




    return STAT_OK;
}

int CKeyPredistribTab::GetDlgData(BASIC_INIT_DATA* pBasicSettings) {
    int     status = STAT_OK;
    
    UpdateData(TRUE);

    pBasicSettings->egScheme.keyPoolSize = (DWORD) atoi(m_keyPoolSize);
    pBasicSettings->egScheme.keyRingSize = (DWORD) atoi(m_keyRingSize);
    pBasicSettings->egScheme.qIntersect = (BYTE) atoi(m_intersectSize);
    pBasicSettings->egScheme.useIt = m_egScheme;
    pBasicSettings->egScheme.maxComprNodes = (DWORD) atoi(m_maxComprNodes);
    pBasicSettings->egScheme.ringRepresentMethod = m_ringAsBitMask ? FLAG_KEYRING_BITMASK : FLAG_KEYRING_IDSET;
    

    pBasicSettings->foeConfig.useLocalBoost = m_useLocalFOEBoost;
    pBasicSettings->foeConfig.useBoostAllDirectNeigh = m_foeBoostAllDirectNeigh;
    pBasicSettings->foeConfig.boostedNodesCount = atol(m_foeBoostCount);
    pBasicSettings->foeConfig.onlySenderTarget = m_notUseFOE;
//    pBasicSettings->foeConfig.assumeSecureLink = m_assumeSecureLinks;
    pBasicSettings->foeConfig.nodesInGroupCount = atol(m_nodesInGroup);
    pBasicSettings->foeConfig.minKeysCount = atol(m_minKeysCount);
    
    
    
    
    return status;
}

int CKeyPredistribTab::SetDlgData(BASIC_INIT_DATA* pBasicSettings) {
    int     status = STAT_OK;
    
    m_keyPoolSize.Format("%d", pBasicSettings->egScheme.keyPoolSize);
    m_keyRingSize.Format("%d", pBasicSettings->egScheme.keyRingSize);
    m_intersectSize.Format("%d", pBasicSettings->egScheme.qIntersect);
    m_egScheme = pBasicSettings->egScheme.useIt;
    m_maxComprNodes.Format("%d", pBasicSettings->egScheme.maxComprNodes);
    m_ringAsBitMask = (pBasicSettings->egScheme.ringRepresentMethod & FLAG_KEYRING_BITMASK) ? TRUE : FALSE;

    m_useLocalFOEBoost = pBasicSettings->foeConfig.useLocalBoost;
    m_foeBoostAllDirectNeigh = pBasicSettings->foeConfig.useBoostAllDirectNeigh;
    m_foeBoostCount.Format("%d", pBasicSettings->foeConfig.boostedNodesCount);
    m_nodesInGroup.Format("%d", pBasicSettings->foeConfig.nodesInGroupCount);
    m_minKeysCount.Format("%d", pBasicSettings->foeConfig.minKeysCount);
    m_notUseFOE = pBasicSettings->foeConfig.onlySenderTarget;
//    m_assumeSecureLinks = pBasicSettings->foeConfig.assumeSecureLink; 

    UpdateData(FALSE);

    return status;
}

void CKeyPredistribTab::OnTestFoe() 
{
    int                 status = STAT_OK;
    BASIC_INIT_DATA     basicSettings;

    // GET SETTINGS AND INIT RANDOM GENERATOR
    if (status == STAT_OK) {
        status = pSensorSim->GetBasicSettings(&basicSettings);
        pSensorSim->m_sensorEnv.InitRandomGenerator(&basicSettings);
    }
    
    if (status == STAT_OK) {
        status = TestFoe(&basicSettings);
    }
}    
    
int CKeyPredistribTab::TestFoe(BASIC_INIT_DATA* pBasicSettings) { 
    int                 status = STAT_OK;
    int                 i;
    FOE_TEST_RESULTS    foeResults;
    FOE_MESSAGE_LIST_NODE*  pMessageProfile = NULL;
    CString             message;
    CString             value;
    DWORD               secured = 0;
    DWORD               compr = 0;
    DWORD               pathsFound = 0;
    DWORD               pathsNotFound = 0;
    INT_DATA_BLOB       sumHopsMsg;
    float               avgComprRatio = 0;

    //
    //  DEPLOY SENSORS AND PREPARE MESSAGE PATHS
    //

    pSensorSim->Log_InsertString("TestFoe start");
    
    // DEPLOY SENSORS AND BASE STATIONS AND 
    if (status == STAT_OK) {
        status = pSensorSim->m_sensorEnv.DeploySensors(pBasicSettings);
        ComputeStatistics(pBasicSettings);
    }

    // PERFORM BASIC WHISPER METHOD TO OBTAIN LINK KEY STATUS WHICH IS EQUAL TO FACT, 
    // THAT MESSAGE TRANSMITTED FROM Ai TO Ai+1 CAN BE CAPTURED BY ATTACKER
    if (status == STAT_OK) {
        if (pBasicSettings->foeConfig.assumeSecureLink) {
            // LINKS ARE ASSUMED SECURE BY DEFINITION, NO SECRECY AMPLIFICATION IS USED
        }
        else {
            if (pBasicSettings->basicWhisper) { 
                status = pSensorSim->m_sensorEnv.BasicWhisper(pBasicSettings, &secured, &compr);
            }
            if (pBasicSettings->mutualWhisper) { 
                status = pSensorSim->m_sensorEnv.MutualWhisper(pBasicSettings, &secured, &compr);
            }
        }
    }

    pSensorSim->Log_InsertString("After deployment");

    // USE ONLY ONE BS PER NODE
    pBasicSettings->bsConfig.bsActive = 1;

    // PREPARE NODES ON PATH FOR EACH POSSIBLE MESSAGE FROM EACH NODE TO BS
    if (status == STAT_OK) {
        pMessageProfile = new FOE_MESSAGE_LIST_NODE;
        status = pSensorSim->m_sensorEnv.FOE_PrepareMessagesPath(pBasicSettings, pMessageProfile, &pathsFound, &pathsNotFound); 
    }

    message.Format("FOE -- secure links: %d | compr. links: %d | foundPaths: %d | notFoundPaths: %d", secured, compr, pathsFound, pathsNotFound);
    LOG_INSERTSTRING(message);

    //
    // PERFORM TESTS WITH VARIABLE NUMBER OF KEYS ETC.
    //
    foeResults.hopsToCaptureCounts.dwMaxLen = 10000; // no more than 10000 hops assumed
    foeResults.hopsToCaptureCounts.pData = new int[foeResults.hopsToCaptureCounts.dwMaxLen];
    memset(foeResults.hopsToCaptureCounts.pData, 0, sizeof(int) * foeResults.hopsToCaptureCounts.dwMaxLen);
    foeResults.hopsToBSCounts.dwMaxLen = 10000; // no more than 10000 hops assumed
    foeResults.hopsToBSCounts.pData = new int[foeResults.hopsToBSCounts.dwMaxLen];
    memset(foeResults.hopsToBSCounts.pData, 0, sizeof(int) * foeResults.hopsToBSCounts.dwMaxLen);
    foeResults.numComprOnHop.dwMaxLen = 10000; // no more than 10000 hops assumed
    foeResults.numComprOnHop.pData = new int[foeResults.numComprOnHop.dwMaxLen];
    memset(foeResults.numComprOnHop.pData, 0, sizeof(int) * foeResults.numComprOnHop.dwMaxLen);
    foeResults.numAppliedKeysCounts.dwMaxLen = pBasicSettings->egScheme.keyRingSize + 1; // no more than keyRingSize keys can be applied
    foeResults.numAppliedKeysCounts.pData = new int[foeResults.numAppliedKeysCounts.dwMaxLen];
    memset(foeResults.numAppliedKeysCounts.pData, 0, sizeof(int) * foeResults.numAppliedKeysCounts.dwMaxLen);

    for (i = 1; i <= pBasicSettings->batchTest.batchRepeat; i++) {
        // CLEAR OLD PREDISTRIB. KEYS
        // PERFORM RANDOM-PREDISTRIB BASED ON SETTINGS
        if (status == STAT_OK) {
            status = pSensorSim->m_sensorEnv.KeyPredistrib_ClearKeys(pBasicSettings);
            status = pSensorSim->m_sensorEnv.KeyPredistrib_EGScheme(pBasicSettings);
        }

        // COMPUTE APPLIED KEYS OVER EACH MESSAGE
        if (status == STAT_OK) {
            status = pSensorSim->m_sensorEnv.FOE_MessageComputeAppliedKeys(pBasicSettings, pMessageProfile);
        }


        if (status == STAT_OK) {
            message.Format("Iteration %d - before FOE_Test", i);
            pSensorSim->Log_InsertString(message);
        
            if ((status = pSensorSim->m_sensorEnv.FOE_Test(pBasicSettings, pMessageProfile, pBasicSettings->egScheme.maxComprNodes, &foeResults)) == STAT_OK) {
                if (i == 1) {
                    message.Format("FOE -- avgHopsToBS: %0.1f | maxHopsToBS: %d | hopsToCapture: %0.1f | avgAppliedKeys: %0.1f | poolSize: %d | ringSize: %d", foeResults.avgHopsToBS, foeResults.maxHopsToBS, foeResults.avgHopsToCapture, foeResults.avgAppliedKeys, pBasicSettings->egScheme.keyPoolSize, pBasicSettings->egScheme.keyRingSize);
                    LOG_INSERTSTRING(message);
                }

                message.Format("FOE -- comprRatio: %0.3f | sec.msg %d | compr.msg %d | compr. nodes: %d | compr. keys: %d", foeResults.comprMsg / (float) (foeResults.secureMsg + foeResults.comprMsg), foeResults.secureMsg, foeResults.comprMsg, foeResults.comprNodes, foeResults.comprKeys);
                LOG_INSERTSTRING(message);

                avgComprRatio += foeResults.comprMsg / (float) (foeResults.secureMsg + foeResults.comprMsg);

    /*
				DWORD               totalMsg;
				int                 j;

                // DETAILED STATISTICS
                sumHopsMsg.dwMaxLen = 10000;
                sumHopsMsg.pData = new int[sumHopsMsg.dwMaxLen];
                memset(sumHopsMsg.pData, 0, sizeof(int) * sumHopsMsg.dwMaxLen);
                for (j = foeResults.maxHopsToBS; j >= 0; j--) {
                    sumHopsMsg.pData[j] = sumHopsMsg.pData[j+1] + foeResults.hopsToBSCounts.pData[j];
                }

                // STATISTICS OF NUMBER OF MESSAGES CAPTURED EXACTLY i HOPS FROM SENDER
                message.Format("Hops capture statistics:\r\n");
                totalMsg = 0;
                for (j = 1; j < foeResults.maxHopsToBS; j++) {
                    if (sumHopsMsg.pData[j] != 0 && foeResults.hopsToCaptureCounts.pData[j] != 0) value.Format("[%d/%d,%d,'%0.2f',%d,'%0.2f']  ", sumHopsMsg.pData[j], foeResults.hopsToBSCounts.pData[j], foeResults.hopsToCaptureCounts.pData[j], foeResults.hopsToCaptureCounts.pData[j] / (float) sumHopsMsg.pData[j], foeResults.numComprOnHop.pData[j], foeResults.numComprOnHop.pData[j]/(float)foeResults.hopsToCaptureCounts.pData[j]);
                    else value.Format("[0/0,%d,'0',%d,'0']  ", foeResults.hopsToCaptureCounts.pData[j], foeResults.numComprOnHop.pData[j]);

                    message += value; 
                    if ((j != 0) && (j % 5) == 0) message += "\r\n";

                    totalMsg += foeResults.hopsToCaptureCounts.pData[j];
                }
                message += "\r\n";
                value.Format("Total captured msg: %d", totalMsg);
                message += value; 
                LOG_INSERTSTRING(message);
    /**/
    /*
                message = "Number of msg. with exactly X keys:\r\n";
                for (j = 0; j < foeResults.numAppliedKeysCounts.dwMaxLen; j++) {
                    value.Format("[%d:%d] ", j, foeResults.numAppliedKeysCounts.pData[j]);

                    message += value; 
                }
                message += "\r\n";
                LOG_INSERTSTRING(message);
/**/
            }
        }
    }


    if (status == STAT_OK) {
        avgComprRatio = avgComprRatio / pBasicSettings->batchTest.batchRepeat;
        message.Format("FOE -- avgComprRatio: %0.3f", avgComprRatio);
        LOG_INSERTSTRING(message);
    }


    // CLEAR PROFILES OF MESSAGES
    while (pMessageProfile != NULL) {
        if (pMessageProfile->prev) {
            pMessageProfile->prev->release();
            delete pMessageProfile->prev;
        }
        pMessageProfile = pMessageProfile->next;
    }

    delete[] foeResults.hopsToCaptureCounts.pData;
    delete[] foeResults.hopsToBSCounts.pData;
    delete[] foeResults.numComprOnHop.pData;
    delete[] sumHopsMsg.pData;
    delete[] foeResults.numAppliedKeysCounts.pData;
    
    
    return status;
}

void CKeyPredistribTab::OnCalcShareProb() 
{
    int     status = STAT_OK;
    double  probKey = 0;
    double  prob;
    DWORD   keys;
    BASIC_INIT_DATA     basicSettings;
    CString message;
    
    UpdateData(TRUE);

    // GET SETTINGS 
    if (status == STAT_OK) {
        status = pSensorSim->GetBasicSettings(&basicSettings);
    }
    
    // COMPUTE BASIC PROBABILITY, THAT NODES WILL SHARE AT LEAST ONE KEYS
    if (status == STAT_OK) {           
        probKey = 0;
        for (keys = 1; keys <= basicSettings.egScheme.keyRingSize; keys++) {
            prob = 0;
            CSensorEnv::EG_SharedKeysProb(basicSettings.egScheme.keyPoolSize, basicSettings.egScheme.keyRingSize, keys, &prob);
            probKey += prob;
            // stop when probability too low (avoid too small numbers)
            if (prob < 0.000000000000000000000000001) break; 
        }
    }

    message.Format("Key share prob %0.4f", probKey);
    LOG_INSERTSTRING(message);
}

void CKeyPredistribTab::OnProbAuthentication() {
    int                 status = STAT_OK;
    BASIC_INIT_DATA     basicSettings;

    // GET SETTINGS AND INIT RANDOM GENERATOR
    if (status == STAT_OK) {
        status = pSensorSim->GetBasicSettings(&basicSettings);
        pSensorSim->m_sensorEnv.InitRandomGenerator(&basicSettings);
    }
    
    if (status == STAT_OK) {
        // SET SPECIFIC SETTINGS FOR EGAUTH SIMULATION 
        basicSettings.foeConfig.assumeSecureLink = TRUE;
        basicSettings.foeConfig.egAuth = TRUE;
    
        status = TestFoe(&basicSettings);
    }
}


void CKeyPredistribTab::OnBnClickedGasNodepick() {
    int                 status = STAT_OK;

    if (pSensorSim->m_sensorEnv.m_pActualDeployment->sensorsDeployed) {
        srand((unsigned int) time(NULL));
        UpdateData(TRUE);
        
        //
        // PREPARE SETTINGS
        //
        //GetGASettings(pSensorSim->m_pGAStrategy);

        // SAVE CONFIGURATION SCRIPT
        pSensorSim->GetBasicSettings(&(pSensorSim->m_sensorEnv.m_basicSettings));

		// SET TEST TYPE FOR THE CASE OF REMOTE EXECUTION
		pSensorSim->m_sensorEnv.m_basicSettings.testType |= FLAG_TESTTYPE_EVOLVENODEPICK;
		
		CString cfgFile; 
        cfgFile.Format("%s\\!cfgScript.ini", pSensorSim->m_basePath);
        status = pSensorSim->m_sensorEnv.SaveConfigScript(cfgFile, &(pSensorSim->m_sensorEnv.m_basicSettings));

	    if (pSensorSim->m_sensorEnv.m_basicSettings.remoteConfig.actualRemoteHost == HOST_ACTUAL) {
			// INTERRUPT PREVIOUS THREAD
			if (pSensorSim->m_pActualThread != NULL) TerminateThread(pSensorSim->m_pActualThread->m_hThread, 0);

			// RESET SOFT FLAG FOR STOPPING CURRENT FLAG
			pSensorSim->m_stopThreadSoft = FALSE;

			// RUN WORKER THREAD FOR EXECUTION
			pSensorSim->m_pActualThread = AfxBeginThread(CKeyPredistribTab::RunEvolveNodePick, NULL, THREAD_PRIORITY_LOWEST); 
			//RunEvolveNodePick(NULL);
		}
		else {
    		// RUN ON SELECTED REMOTE COMPUTER
			pSensorSim->RunRemoteExecution(pSensorSim->m_sensorEnv.m_basicSettings.remoteConfig.actualRemoteHost, cfgFile, "XX2");   
		}
    }
    else {
        pSensorSim->Log_InsertString("Network not deployed!", FALSE); 
    }
    
}

UINT CKeyPredistribTab::RunEvolveNodePick(LPVOID pParam) {
	UINT     status = STAT_OK;
	//
    // ALLOCATE GENOM AND GA ALGORITHM, ONLY IF NOT PREPARED BEFORE (E.G. LOADED FROM FILE)
    //
    GARandomSeed();
    BOOL    bNewPopulation = FALSE;
    // GENOM LENGTH WILL BE EQUAL TO NUMBER OF CAPTURED NODES
    pSensorSim->m_pGAStrategy->gLength = pSensorSim->m_sensorEnv.m_basicSettings.egScheme.maxComprNodes;
    //ASSERT(pSensorSim->m_pGAStrategy->gLength == 50);
    if (pSensorSim->m_pGAStrategy->pGA == NULL) {
        pSensorSim->m_pGAStrategy->pGenom = new GA1DArrayGenome<NCR_ELEMENT_TYPE> (pSensorSim->m_pGAStrategy->gLength, CSensorEnv::NCR_fitness);
        pSensorSim->m_pGAStrategy->pGenom->initializer(CSensorEnv::NCR_initializer_best); 
        pSensorSim->m_pGAStrategy->pGenom->mutator(CSensorEnv::NCR_mutator); 
        pSensorSim->m_pGAStrategy->pGenom->crossover(CSensorEnv::NCR_crossover); 
                    
        pSensorSim->m_pGAStrategy->pGA = new GASteadyStateGA(*(pSensorSim->m_pGAStrategy->pGenom));
        // FLAG AS NEW POPULATION
        bNewPopulation = TRUE;
    }        
        
    pSensorSim->m_pGAStrategy->pGA->nReplacement(2 * (pSensorSim->m_pGAStrategy->popSize / 3));  
    pSensorSim->m_pGAStrategy->pGA->populationSize(pSensorSim->m_pGAStrategy->popSize); 
    pSensorSim->m_pGAStrategy->pGA->nGenerations(pSensorSim->m_pGAStrategy->nGeners);
    pSensorSim->m_pGAStrategy->pGA->pCrossover(pSensorSim->m_pGAStrategy->pCross);
    pSensorSim->m_pGAStrategy->pGA->pMutation(pSensorSim->m_pGAStrategy->pMutt); 


    // MAKE 5 RANDOM TEST SELECTION
    CString message;
    float   fit = 0;
    float   randomAvg = 0;
    for (int i = 1; i < 6; i++) {
        CSensorEnv::NCR_initializer_random(*pSensorSim->m_pGAStrategy->pGenom);
        fit = CSensorEnv::NCR_fitness(*pSensorSim->m_pGAStrategy->pGenom);
        message.Format("Random%d: %f", i, fit);
        LOG_INSERTSTRING(message);
        randomAvg += fit;
    }
    message.Format("# Random avg: %f", randomAvg / 5);
    LOG_INSERTSTRING(message);
    
    // TEST ALG PICK
    fit = CSensorEnv::NCR_PickAlg(0);
    message.Format("MinPrice: %f", fit);
    LOG_INSERTSTRING(message);
    fit = CSensorEnv::NCR_PickAlg(1);
    message.Format("MaxPrice: %f", fit);
    LOG_INSERTSTRING(message);
/*
    fit = CSensorEnv::NCR_PickAlg2(0);
    message.Format("KeySet min price: %f", fit);
    LOG_INSERTSTRING(message);
/**/    
    fit = CSensorEnv::NCR_PickAlg2(1);
    message.Format("# KeySet max price:: %f", fit);
    LOG_INSERTSTRING(message);
    fit = CSensorEnv::NCR_PickAlg3(1, 0);
    message.Format("# KeySet link max price:: %f", fit);
    LOG_INSERTSTRING(message);
    message = "";
    for (int i = 0; i < 50; i++) {
        CString value;
        value.Format("%d ", pSensorSim->m_sensorEnv.bestNC[i]);
        message += value;
    }
    LOG_INSERTSTRING(message);
    fit = CSensorEnv::NCR_PickAlg3(1, 1);
    message.Format("# KeySet link max price (1 rand): %f", fit);
    LOG_INSERTSTRING(message);
    message = "";
    for (int i = 0; i < 50; i++) {
        CString value;
        value.Format("%d ", pSensorSim->m_sensorEnv.bestNC[i]);
        message += value;
    }
    LOG_INSERTSTRING(message);
    fit = CSensorEnv::NCR_PickAlg3(1, 10);
    message.Format("# KeySet link max price (10 rand): %f", fit);
    LOG_INSERTSTRING(message);
    message = "";
    for (int i = 0; i < 50; i++) {
        CString value;
        value.Format("%d ", pSensorSim->m_sensorEnv.bestNC[i]);
        message += value;
    }
    LOG_INSERTSTRING(message);
    memset(pSensorSim->m_sensorEnv.bestNC, 0, sizeof(pSensorSim->m_sensorEnv.bestNC));    
    
    // INITIALIZE POPULATION
    if (bNewPopulation) {
        pSensorSim->m_pGAStrategy->pGA->initialize();
    }

    
    // 
    //  RUN EVOLUTION
    //
    status = pSensorSim->m_sensorEnv.NCR_Evolve(pSensorSim->m_pGAStrategy);
    
    //  RELEASE STRUCTS
    if (pSensorSim->m_pGAStrategy->pGenom) {
        delete pSensorSim->m_pGAStrategy->pGenom;
        pSensorSim->m_pGAStrategy->pGenom = NULL;
    }       
    if (pSensorSim->m_pGAStrategy->pGA) {
        delete pSensorSim->m_pGAStrategy->pGA;
        pSensorSim->m_pGAStrategy->pGA = NULL;
    }

/*    
    // DELETE THREAD PARAM STRUCT
    delete pThreadParam;
/**/
    // SET ACTIVE THREAD PTR TO NULL
    pSensorSim->m_pActualThread = NULL;
    
    return status;
}
   