#include "stdafx.h"
#include "SensorSim.h"
#include "SensorSimDlg.h"
#include "ScriptAction.h"

extern CSensorSimDlg*  pSensorSim;

// ***************************************************************************************
CNewKeyAction::CNewKeyAction(int keyPtrIndex) {
    ppKey = NULL;
    ppKeyPtrIndex = keyPtrIndex;
}
int CNewKeyAction::TryAssignPointer(PTR_TYPE ptrType, int ptrIndex, void* pPtr) {
    if (ptrType == SENSITIVE_OBJECT_Ptr) {
        if (ppKeyPtrIndex == ptrIndex) ppKey = (SENSITIVE_OBJECT**) pPtr;
    }
    return STAT_OK;
}

int CNewKeyAction::Execute() {
    int     status;
    if ((status = CheckPointerCorrectness()) == STAT_OK) {
        status = pSensorSim->m_sensorEnv.SS_NewKeyObject(FLAG_SENSITOBJECT_SIMPLEKEY, ppKey);                       
    }
    return status;
}

int CNewKeyAction::CheckPointerCorrectness() {
    if (ppKey == NULL) return STAT_INVALID_PTR;
    else return STAT_OK;
}

// ***************************************************************************************
CAddKeyAction::CAddKeyAction(int keyPtrIndex, int sensorPtrIndex) {
    ppKeyPtrIndex = keyPtrIndex;
    pSensorPtrIndex = sensorPtrIndex;
    ppKey = NULL;
    pSensor = NULL;
}

int CAddKeyAction::TryAssignPointer(PTR_TYPE ptrType, int ptrIndex, void* pPtr) {
    if (ptrType == SENSITIVE_OBJECT_Ptr) {
        if (ppKeyPtrIndex == ptrIndex) ppKey = (SENSITIVE_OBJECT**) pPtr;
    }
    if (ptrType == SENSOR_NODE_Ptr) {
        if (pSensorPtrIndex == ptrIndex) pSensor = (SENSOR_NODE*) pPtr;
    }
    return STAT_OK;
}

int CAddKeyAction::Execute() {
    int     status;

    if ((status = CheckPointerCorrectness()) == STAT_OK) {
        // IF KEY IS COMPOSITE, THEN CHECK FOR PRESENCE OF ALL COMPONENT KEYS
        if ((*ppKey)->typeFlags & FLAG_SENSITOBJECT_COMPOSITEKEY) {
            if (!pSensor->ss_haveAllKeys((*ppKey)->components)) {
                status = STAT_KEY_MISSING;
            }
        }

        if (status == STAT_OK) {
            status = pSensor->ss_addKey(*ppKey);
        }
    }

    return status;
}
int CAddKeyAction::CheckPointerCorrectness() {
    if (ppKey == NULL || *ppKey == NULL || pSensor == NULL) return STAT_INVALID_PTR;
    else return STAT_OK;
}

// ***************************************************************************************
CCombineKeysAction::CCombineKeysAction(int key1PtrIndex, int key2PtrIndex, int keyCompositePtrIndex) {
    ppKey1PtrIndex = key1PtrIndex;
    ppKey2PtrIndex = key2PtrIndex;
    ppKeyCompositePtrIndex = keyCompositePtrIndex;
    ppKey1 = NULL;
    ppKey2 = NULL;
    ppKeyComposite = NULL;
}

int CCombineKeysAction::TryAssignPointer(PTR_TYPE ptrType, int ptrIndex, void* pPtr) {
    if (ptrType == SENSITIVE_OBJECT_Ptr) {
        if (ppKey1PtrIndex == ptrIndex) ppKey1 = (SENSITIVE_OBJECT**) pPtr;
        if (ppKey2PtrIndex == ptrIndex) ppKey2 = (SENSITIVE_OBJECT**) pPtr;
        if (ppKeyCompositePtrIndex == ptrIndex) ppKeyComposite = (SENSITIVE_OBJECT**) pPtr;
    }
    return STAT_OK;
}

int CCombineKeysAction::Execute() {
    int                 status;

    if ((status = CheckPointerCorrectness()) == STAT_OK) {
        if ((status = pSensorSim->m_sensorEnv.SS_CombineKeys2(*ppKey1, *ppKey2, ppKeyComposite)) == STAT_OK) {
        }
    }
    return status;
}
int CCombineKeysAction::CheckPointerCorrectness() {
    if (ppKey1 == NULL || *ppKey2 == NULL || *ppKeyComposite == NULL) return STAT_INVALID_PTR;
    else return STAT_OK;
}

// ***************************************************************************************
CNewMessageAction_noKey::CNewMessageAction_noKey(int contentPtrIndex, int messagePtrIndex) {
    ppContentPtrIndex = contentPtrIndex;
    ppMessagePtrIndex = messagePtrIndex;
    ppContent = NULL;
    ppMessage = NULL;
}

int CNewMessageAction_noKey::TryAssignPointer(PTR_TYPE ptrType, int ptrIndex, void* pPtr) {
    if (ptrType == SENSITIVE_OBJECT_Ptr) {
        if (ppContentPtrIndex == ptrIndex) ppContent = (SENSITIVE_OBJECT**) pPtr;
        if (ppMessagePtrIndex == ptrIndex) ppMessage = (SENSITIVE_OBJECT**) pPtr;
    }
    return STAT_OK;
}

int CNewMessageAction_noKey::Execute() {
    int     status;
    if ((status = CheckPointerCorrectness()) == STAT_OK) {
        status = pSensorSim->m_sensorEnv.SS_NewMessageObject(NULL, *ppContent, ppMessage);
    }
    return status;
}
int CNewMessageAction_noKey::CheckPointerCorrectness() {
    if (*ppContent == NULL || ppMessage == NULL) return STAT_INVALID_PTR;
    else return STAT_OK;
}

// ***************************************************************************************
CNewMessageAction_oneKey::CNewMessageAction_oneKey(int contentPtrIndex, int keyPtrIndex, int messagePtrIndex) {
    ppContentPtrIndex = contentPtrIndex;
    ppKeyPtrIndex = keyPtrIndex;
    ppMessagePtrIndex = messagePtrIndex;
    ppContent = NULL;
    ppKey = NULL;
    ppMessage = NULL;
}

int CNewMessageAction_oneKey::TryAssignPointer(PTR_TYPE ptrType, int ptrIndex, void* pPtr) {
    if (ptrType == SENSITIVE_OBJECT_Ptr) {
        if (ppContentPtrIndex == ptrIndex) ppContent = (SENSITIVE_OBJECT**) pPtr;
        if (ppKeyPtrIndex == ptrIndex) ppKey = (SENSITIVE_OBJECT**) pPtr;
        if (ppMessagePtrIndex == ptrIndex) ppMessage = (SENSITIVE_OBJECT**) pPtr;
    }
    return STAT_OK;
}

int CNewMessageAction_oneKey::Execute() {
    int     status;
    if ((status = CheckPointerCorrectness()) == STAT_OK) {
        status =  pSensorSim->m_sensorEnv.SS_NewMessageObject(*ppKey, *ppContent, ppMessage);
    }
    return status;
}
int CNewMessageAction_oneKey::CheckPointerCorrectness() {
    if (ppContent == NULL || *ppContent == NULL || ppKey == NULL || *ppKey == NULL || ppMessage == NULL) return STAT_INVALID_PTR;
    else return STAT_OK;
}

// ***************************************************************************************
CSendMessageAction::CSendMessageAction(int senderPtrIndex, int targetPtrIndex, int messagePtrIndex) {
    pSenderNodePtrIndex = senderPtrIndex;
    pTargetNodePtrIndex = targetPtrIndex;
    ppMessagePtrIndex = messagePtrIndex;
    pSenderNode = NULL;
    pTargetNode = NULL;
    ppMessage = NULL;
}

int CSendMessageAction::TryAssignPointer(PTR_TYPE ptrType, int ptrIndex, void* pPtr) {
    if (ptrType == SENSITIVE_OBJECT_Ptr) {
        if (ppMessagePtrIndex == ptrIndex) ppMessage = (SENSITIVE_OBJECT**) pPtr;
    }
    if (ptrType == SENSOR_NODE_Ptr) {
        if (pSenderNodePtrIndex == ptrIndex) pSenderNode = (SENSOR_NODE*) pPtr;
        if (pTargetNodePtrIndex == ptrIndex) pTargetNode = (SENSOR_NODE*) pPtr;
    }

    return STAT_OK;
}

int CSendMessageAction::Execute() {
    int     status;
    if ((status = CheckPointerCorrectness()) == STAT_OK) {
        status = pSensorSim->m_sensorEnv.SS_SendMessage(pSenderNode, pTargetNode, *ppMessage);
    }
    return status;
}
int CSendMessageAction::CheckPointerCorrectness() {
    if (pSenderNode == NULL || pTargetNode == NULL || ppMessage == NULL || *ppMessage == NULL) return STAT_INVALID_PTR;
    else return STAT_OK;
}

// ***************************************************************************************
CDispatchMessageAction::CDispatchMessageAction(int nodePtrIndex) {
    pNodePtrIndex = nodePtrIndex;
    pNode = NULL;
}

int CDispatchMessageAction::TryAssignPointer(PTR_TYPE ptrType, int ptrIndex, void* pPtr) {
    if (ptrType == SENSOR_NODE_Ptr) {
        if (pNodePtrIndex == ptrIndex) pNode = (SENSOR_NODE*) pPtr;
    }

    return STAT_OK;
}

int CDispatchMessageAction::Execute() {
    int     status;
    if ((status = CheckPointerCorrectness()) == STAT_OK) {
        status = pNode->ss_dispatchMessages();
    }
    return status;
}
int CDispatchMessageAction::CheckPointerCorrectness() {
    if (pNode == NULL) return STAT_INVALID_PTR;
    else return STAT_OK;
}
