// SensorSimDlg.h : header file
//

#if !defined(AFX_SENSORSIMDLG_H__092779F6_3178_4807_A935_2E20BA1CFFD6__INCLUDED_)
#define AFX_SENSORSIMDLG_H__092779F6_3178_4807_A935_2E20BA1CFFD6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BasicTab.h"
#include "KeyEstablishTab.h"
#include "KeyInfectTab.h"
#include "KeyPredistribTab.h"
#include "BSRouteTab.h"
#include "MeshTab.h"
#include "NeighLeashesTab.h"
#include "RemoteControlTab.h"
#include "GraphicTab.h"
#include "SensorEnv.h"
#include "socket.h"
#include "EACircuit.h"

/////////////////////////////////////////////////////////////////////////////
// CSensorSimDlg dialog

class CSensorSimDlg : public CDialog
{
// Construction
public:
	CSensorSimDlg(CWnd* pParent = NULL);	// standard constructor
    ~CSensorSimDlg();
// Dialog Data
	//{{AFX_DATA(CSensorSimDlg)
	enum { IDD = IDD_SENSORSIM_DIALOG };
	CTabCtrl	m_mainTab;
	CString	    m_logs;
	BOOL	    m_keepLogBelow;
	CString	    m_maxLogSize;
	CString     m_basePath;
	//}}AFX_DATA
    GA_STRATEGY*     m_pGAStrategy;   
    GA_STRATEGY		 m_tempStrategy; // temporary strategy structure

    
    int                             perfNestLevel;      // actual nesting level of performance monitor
    lcs                             perfTimeList;       // results for performance monitor
    CString                         perfTimeString;     // help string for formating
    msd                             perfCallTime;       // map that counts total time in given function of function ivocations (by fnc name)
    msd                             perfCallCounter;    // map that can count number of function ivocations (by fnc name)
    
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CSensorSimDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
	//}}AFX_VIRTUAL

public:
//    BOOL        m_pActualDeployment->sensorsDeployed;
    BOOL        m_stopThreadSoft;
    CWinThread* m_pActualThread;


// Implementation
public:
    //
    //  LOGGING FUNCTIONS
    //
    int Log_InsertString(CString message, BOOL bShowTime = TRUE, BOOL bToFile = TRUE);
    int Log_AppendString(CString message);
    int Log_InsertClearLine();
    int Log_InsertSeparator();
    int Log_EnsureVisibleLastLine();
    
    int Log_InsertToFile(CString fileName, CString message);
    
    void ClearPerformance();
    void ShowPerformance();
 
    int GetBasicSettings(BASIC_INIT_DATA* pBasicSettings);
    int SetBasicSettings(BASIC_INIT_DATA* pBasicSettings);
    int GetBasicSettingsHeader(BASIC_INIT_DATA* pBasicSettings);

    void RunProxy(int port);
	int SocketSendLine(SocketClient* pSocket, CString line, CString* pResponse, CString* pStatus = NULL);
    
    int RunRemoteExecution(CString host, CString configFilePath, CString instanceSubdir);

    CString             m_logFile;
	CString				m_defaultCfgFile;
    CSensorEnv          m_sensorEnv;        // class with deployed sensors structures
    
    CEACircuit*         m_pEACircuit;       // class with structures and functions for evolution curcuit    
    
protected:
    SocketServer*       m_pSocketServer;
    CWinThread*         m_pProxyThread;  
public:
    int                 m_numSocketConnections;    
    BOOL                m_bStopThread;    
protected:

    void HideAllTabs();
    void ShowBasicTab();
    void ShowKeyEstablishTab();
    void ShowKeyInfectTab();
    void ShowKeyPredistribTab();
    void ShowBSRouteTab();
    void ShowMeshTab();
    void ShowNeighLeashTab();
	void ShowRemoteControlTab();
	void ShowGraphicTab();
	void ShowSpecificTab(int tabID);


public:
    CBasicTab           m_tabBasic;
    CKeyEstablishTab    m_tabKeyEstablish;
    CKeyInfectTab       m_tabKeyInfect;
    CKeyPredistribTab   m_tabKeyPredistrib;
    CBSRouteTab         m_tabBSRoute;
    CMeshTab            m_tabMesh;
    CNeighLeashesTab    m_tabNeighLeash;
	CRemoteControlTab	m_tabRemoteControl;
	CGraphicTab			m_tabGraphic;
protected:    
	HICON m_hIcon;

	// Generated message map functions
	//{{AFX_MSG(CSensorSimDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg void OnSelchangeMaintab(NMHDR* pNMHDR, LRESULT* pResult);
    afx_msg LRESULT OnLogInsertString(WPARAM wParam, LPARAM lParam = TRUE);
    afx_msg LRESULT OnUpdateRNGSeed(WPARAM wParam, LPARAM lParam = TRUE);
	afx_msg void OnClose();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
public:
    afx_msg void OnEnChangeLogs();
public:
    afx_msg void OnBnClickedStopCurrentOperations();
public:
    CString m_hostPC;
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_SENSORSIMDLG_H__092779F6_3178_4807_A935_2E20BA1CFFD6__INCLUDED_)
